@@ -4,17 +4,13 @@
 
 import pandas as pd
 import numpy as np
-import datetime, time
-import argparse
+import time
 import itertools
-import pickle
-import io
 import sys
 # from tqdm import tqdm
 from tqdm.auto import tqdm
 from os import path, makedirs, remove
 from difflib import get_close_matches
-import json
 
 from .globals_ import OUTPUT_DIR, OUTPUT_FOLDER, DATA_DIR
 from .aaindex import  AAIndex
@@ -24,12 +20,11 @@ from .evaluate import Evaluate
 from .pySAR import PySAR
 from .utils import *
 from .descriptors_ import Descriptors
-from .plots import plot_reg
 
 class Encoding(PySAR):
     """
     The use-case of this class is when you have a dataset of protein sequences with
-    a sought-after protein activity value and you want to measure this activity
+    a sought-after protein activity/fitness value and you want to measure this activity
     value for new and unseen sequences that have not had their activity value
     experimentally measured. The encoding class allows for evaluation of a variety
     of potential techniques at which to numerically encode the protein sequences,
@@ -41,29 +36,30 @@ class Encoding(PySAR):
     The encoding class inherits from the main PySAR module and allows for a
     dataset of protein sequences to be encoded through 3 main strategies: AAI Indices,
     protein Descriptors and AAI Indices + protein Descriptors. The encoding class
-    and its methods differ from the PySAR class by allowing for the encoding of
+    and its methods differ from the PySAR class by allowing for the encoding using
     all available features. To date, there are 566 indices supported in the AAI and
     pySAR supports 15 different descriptors. The features can be encoeed using
     different combinations, for example, 1 2 or 3 descriptors can be used for the
     descriptor and AAI + Descriptor encoding strategies. In total, this class
     supports over 410,000 possible ways at which to numerically encode the
-    protein sequences in the building of a predictive ML model.
+    protein sequences in the building of a predictive ML model for mapping these
+    sequences to a particular activity, known as a Sequence-Activity-Relationship (SAR).
 
     Attributes
     ----------
-        :config_file (str): path to configuration file.
+        :config_file (str): 
+            path to configuration file.
 
     Methods
     -------
     aai_encoding(self, aai_list=None, cutoff_index=1):
         encoding protein sequences using indices from the AAI.
     descriptor_encoding(desc_combo=1):
-        encoding protein sequences using protein descriptors from descrtipors module.
+        encoding protein sequences using protein descriptors from descriptors module.
     aai_descriptor_encoding(desc_combo=1, use_dsp=True):
         encoding protein sequences using indices from the AAI in concatenation
         with the protein descriptors from the descriptors module.
     """
-
     def __init__(self, config_file=""):
 
         self.config_file = conf= config_file
@@ -87,12 +83,12 @@ class Encoding(PySAR):
 
         Parameters
         ----------
-        :aai_list : list (default = None)
+        :aai_list : list (default=None)
             list of aai indices to use for encoding the predictive models, by default
             all AAI indices will be used.
-        :cutoff_index : int (default = 1)
+        :cutoff_index : int (default=1)
             set the proportion of iterations of AAI indices to actually encode -> mainly used for testing.
-        :sort_by : str (default = R2)
+        :sort_by : str (default=R2)
             sort output dataframe by specified column/metric value, results sorted by R2 score by default.
 
         Returns
@@ -109,11 +105,10 @@ class Encoding(PySAR):
             self.convolution = None
 
         #initialise dataframe to store all output results of AAI encoding
-        aaindex_metrics_df = pd.DataFrame(columns=['Index','Category','R2', 'RMSE',
+        aaindex_metrics_df = pd.DataFrame(columns=['Index', 'Category', 'R2', 'RMSE',
             'MSE', 'RPD', 'MAE', 'Explained Variance'])
 
         #lists to store results for each predictive model
-        aa_list = []
         index_ = []
         category_ = []
         r2_ = []
@@ -158,7 +153,7 @@ class Encoding(PySAR):
 
         #using tqdm package to create a progress bar showing encoding progress
         #   file=sys.stdout to stop error where iterations were printing out of order
-        for index in tqdm(all_indices[:cutoff_index],unit=" indices",position=0,desc="AAI Indices",file=sys.stdout):
+        for index in tqdm(all_indices[:cutoff_index], unit=" indices", position=0, desc="AAI Indices", file=sys.stdout):
 
             #get AAI indices encoding for sequences according to index var
             encoded_seqs = self.get_aai_encoding(index)
@@ -217,10 +212,10 @@ class Encoding(PySAR):
             sort_by = 'R2'
 
         #sort results by R2
-        aaindex_metrics_ = aaindex_metrics_.sort_values(by=[sort_by],ascending=False)
+        aaindex_metrics_ = aaindex_metrics_.sort_values(by=[sort_by], ascending=False)
 
         #save results dataframe, saved to OUTPUT_DIR by default.
-        save_results(aaindex_metrics_,'aaindex_results')
+        save_results(aaindex_metrics_, 'aaindex_results')
 
         return aaindex_metrics_
 
@@ -233,30 +228,31 @@ class Encoding(PySAR):
         descriptor encoding will be used as the feature data to build the predictive
         regression models. With 15 descriptors supported by pySAR this means there
         can be 15, 105 and 455 total predictive models built for 1, 2 or 3 descriptors,
-        respecitvely. The metrics evaluated from the model for each Descriptor encoding
+        respecitvely. These totals may vary depending on the meta-parameters of each of 
+        the descriptors. The metrics evaluated from the model for each descriptor encoding
         combination will be collated into a dataframe and saved and returned, with the 
         results sorted by the R2 score, this can be changed using the sort_by parameter.
 
         Parameters
         ----------
-        :desc_combo : int (default = 1)
+        :desc_combo : int (default=1)
             combination of descriptors to use.
-        :decs_list : list (default = None)
+        :decs_list : list (default=None)
             list of descriptors to use for encoding, by default all available descriptors
             in the descriptors module will be used for the encoding.
-        :cutoff_index : int (default = 1)
+        :cutoff_index : int (default=1)
             set the proportion of iterations of descriptors to actually encode -> mainly used for testing.
-        :sort_by : str (default = R2)
+        :sort_by : str (default=R2)
             sort output dataframe by specified column/metric value, results sorted by R2 score by default.
 
         Returns
         -------
         :desc_metrics_df_ : pd.DataFrame
             dataframe of calculated metric values from generated predictive models
-            encoded using all descriptors for the descriptors encoding strategy.
+            encoded using all or selected descriptors for the descriptors encoding strategy.
         """
         #create dataframe to store output results from models
-        desc_metrics_df = pd.DataFrame(columns=['Descriptor','Group','R2', 'RMSE',
+        desc_metrics_df = pd.DataFrame(columns=['Descriptor', 'Group', 'R2', 'RMSE',
             'MSE', 'RPD', 'MAE', 'Explained Variance'])
 
         #lists to store results for each predictive model
@@ -272,9 +268,8 @@ class Encoding(PySAR):
         desc_count = 1  #counters to keep track of current index & descriptor
 
         #if no descriptors passed into desc_list then use all descriptors by default,
-        #   get list of  all descriptors according to desc_combo value
+        #   get list of all descriptors according to desc_combo value
         desc = Descriptors(self.config_file)
-        # print('ctd here', desc.ctd)
         if desc_list == None or desc_list == [] or desc_list == "":
             # desc = Descriptors(self.config_file)
             all_descriptors = desc.all_descriptors_list(desc_combo)
@@ -309,7 +304,7 @@ class Encoding(PySAR):
         5.) Repeat steps 1 - 4 for all descriptors.
         6.) Output results into a final dataframe, save it and return, sorting by sort_by parameter.
         '''
-        for descr in tqdm(all_descriptors[:cutoff_index],unit=" descriptor",position=0,desc="Descriptors",file=sys.stdout):
+        for descr in tqdm(all_descriptors[:cutoff_index], unit=" descriptor", position=0, desc="Descriptors", file=sys.stdout):
 
             desc_ = pd.DataFrame()           #reset descriptor DF and list
             descriptor_list = []
@@ -322,33 +317,28 @@ class Encoding(PySAR):
                     descriptor_group_.append(desc.descriptor_groups[de])
                 desc_ = pd.concat(descriptor_list,axis=1) #concatenate descriptors
             else:
-                # desc_ = getattr(desc, descr)
                 desc_ = desc.get_descriptor_encoding(descr)
                 descriptor_group_.append(desc.descriptor_groups[descr])
-                # descriptor_group_.append(desc.descriptor_groups[descr])
 
             X = desc_   #set training data to desc_ dataframe
 
             #get protein activity values
             Y  = self.get_activity()
-            print(descr)
-            print(X)
-            # print(Y.shape)
             '''
             If using the PlsRegression algorithm and there is only 1 feature (1-dimension)
-            in the feature data X then create a new PLSReg model with the n_components
+            in the feature data X (e.g SOCNum) then create a new PLSReg model with the n_components
             parameter set to 1 instead of the default 2 - this stops the error:
             'ValueError - Invalid Number of Components: 2'
 
             Also, get train/test split, fit model and predict activity of test data.
             '''
             if X.shape[1] == 1 and repr(self.model) == "PLSRegression":
-              tmp_model = Model('plsreg',parameters={'n_components':1})
-              X_train, X_test, Y_train, Y_test  = tmp_model.train_test_split(X,Y,test_size=self.test_split)
+              tmp_model = Model('plsreg', parameters={'n_components':1})
+              X_train, X_test, Y_train, Y_test  = tmp_model.train_test_split(X, Y, test_size=self.test_split)
               model_fit = tmp_model.fit()
               Y_pred = tmp_model.predict()
             else:
-              X_train, X_test, Y_train, Y_test  = self.model.train_test_split(X,Y,test_size=self.test_split)
+              X_train, X_test, Y_train, Y_test  = self.model.train_test_split(X, Y, test_size=self.test_split)
               model_fit = self.model.fit()
               Y_pred = self.model.predict()
 
@@ -394,7 +384,7 @@ class Encoding(PySAR):
             sort_by = 'R2'
 
         #sort results by R2
-        desc_metrics_df_ = desc_metrics_df_.sort_values(by=[sort_by],ascending=False)
+        desc_metrics_df_ = desc_metrics_df_.sort_values(by=[sort_by], ascending=False)
 
         #set save path according to the descriptor combinations type
         if desc_combo == 2:
@@ -409,20 +399,21 @@ class Encoding(PySAR):
 
         return desc_metrics_df_
 
-    def aai_descriptor_encoding(self, aai_list=None,desc_list=None,desc_combo=1,cutoff_index=1,sort_by='R2'):
+    def aai_descriptor_encoding(self, aai_list=None, desc_list=None, desc_combo=1, cutoff_index=1, sort_by='R2'):
         """
         Encoding all protein sequences using each of the indices in the AAI as well
         as the protein descriptors. The sequences can be encoded using 1 AAI + 1 Descriptor,
         2 Descriptors or 3 Descriptors, dictated by the desc_combo input parameter:
-        set this to 1,2 or 3 for what encoding combination to use, default is 1.
-        The protein spectra of the AAI indices will be generated if use_dsp is true,
-        dictated by the class attributes: spectrum, window and filter.
+        set this to 1, 2 or 3 for what encoding combination to use, default is 1.
+        The protein spectra of the AAI indices will be generated if the config param 
+        use_dsp is true, along with the class attributes: spectrum, window, convolution and filter.
         Each encoding will be used as the feature data to build the predictive
         regression models. To date, there are 566 indices and pySAR supports 15
         descriptors so the encoding process will generate 8490, ~59000 and ~257000
-        models, when using 1, 2 or 3 descriptors + AAI indices, respectively. The
-        metrics evaluated from the model for each AAI + Descriptor encoding combination
-        will be collated into a dataframe and saved and returned, sorted by R2 score by default.
+        models, when using 1, 2 or 3 descriptors + AAI indices, respectively. These values
+        may vary depending on the meta-parameters of each descriptor. The metrics evaluated 
+        from the model for each AAI + Descriptor encoding combination will be collated into a 
+        dataframe and saved and returned, sorted by R2 score by default.
 
         Parameters
         ----------
@@ -451,8 +442,8 @@ class Encoding(PySAR):
             self.convolution = None
 
         #create dataframe to store output results from models
-        aai_desc_metrics_df = pd.DataFrame(columns=['Index','Category', 'Descriptor',\
-            'Descriptor Group','R2', 'RMSE', 'MSE', 'RPD', 'MAE', 'Explained Variance'])
+        aai_desc_metrics_df = pd.DataFrame(columns=['Index', 'Category', 'Descriptor',\
+            'Descriptor Group', 'R2', 'RMSE', 'MSE', 'RPD', 'MAE', 'Explained Variance'])
 
         #lists to store results for each predictive model
         index_ = []
@@ -471,7 +462,7 @@ class Encoding(PySAR):
 
         #if no indices passed into aai_list then use all indices by default
         if aai_list == None or aai_list == [] or aai_list == "":
-            all_indices = self.aaindex.get_record_codes()
+            all_indices = self.aaindex.record_codes()
         elif isinstance(aai_list, str):   #if single descriptor input, cast to list
             all_indices = [aai_list]
         else:
@@ -479,17 +470,15 @@ class Encoding(PySAR):
 
         #if no descriptors passed into desc_list then use all descriptors by default,
         #   get list of all descriptors according to desc_combo value
+        desc = Descriptors(self.config_file)
         if desc_list == None or desc_list == [] or desc_list == "":
-            # desc = Descriptors(self.data[self.sequence_col], all_desc = True)
-            desc = Descriptors(self.config_file)
+            # desc = Descriptors(self.config_file)
             all_descriptors = desc.all_descriptors_list(desc_combo)
-        elif isinstance(desc_list,str):     #if single descriptor input, cast to list
-            # desc = Descriptors(self.data[self.sequence_col])
-            desc = Descriptors(self.config_file)
+        elif isinstance(desc_list, str):     #if single descriptor input, cast to list
+            # desc = Descriptors(self.config_file)
             all_descriptors = [desc_list]
         else:
-            # desc = Descriptors(self.data[self.sequence_col])
-            desc = Descriptors(self.config_file)
+            # desc = Descriptors(self.config_file)
             if desc_combo == 2:
                 all_descriptors = list(itertools.combinations(desc_list, 2))
             elif desc_combo == 3:
@@ -499,9 +488,16 @@ class Encoding(PySAR):
 
         print('\n\n##############################################################\n')
         print('Encoding using {} AAI and {} descriptor combination(s) with the parameters:\n\
-            \n# Window -> {}\n# Filter -> {}\n# Spectrum -> {}\n# Convolution -> {}\n# Algorithm -> {}\n# Parameters -> {}\
-            \n# Test Split -> {}\n'.format(len(all_indices), len(all_descriptors),
-                self.window, self.filter_, self.spectrum, self.convolution, repr(self.model), self.parameters, self.test_split))
+            \n# Algorithm -> {}\n# Parameters -> {}\n# Test Split -> {}\n'.
+                format(len(all_indices), len(all_descriptors), repr(self.model), self.parameters, self.test_split))
+        if ((len(all_indices)<10) and (len(all_indices)>0)):
+            print('# AAI Indices -> {}\n'.format(len(all_indices)))
+            if (self.use_dsp):
+                print('  # Using DSP -> {}\n  # Spectrum -> {}\n  # Window Function -> {} \
+                \n  # Filter Function -> {}\n  # Convolution -> {}'.format(
+                    self.use_dsp, self.spectrum, self.window, self.filter, self.convolution)) 
+        if ((len(all_descriptors)<10) and (len(all_descriptors)>0)):
+            print('# Descriptors -> {}\n'.format(len(all_descriptors)))
         print('##################################################################\n')
 
         #set the proportion of iterations to actually complete -> mainly used for testing
@@ -520,13 +516,13 @@ class Encoding(PySAR):
         6.) Repeat steps 1 - 5 for all indices in the AAI.
         7.) Output results into a final dataframe, save it and return, sort by sort_by parameter.
         '''
-        for index in tqdm(all_indices[:cutoff_index],unit=" indices",desc="AAI Indices"):
+        for index in tqdm(all_indices[:cutoff_index], unit=" indices", desc="AAI Indices"):
 
             #get AAI indices encoding for sequences according to index var
             encoded_seqs = self.get_aai_encoding(index)
 
             #generate protein spectra from pyDSP class if use_dsp is true
-            if use_dsp:
+            if self.use_dsp:
                 pyDSP = PyDSP(self.config_file, protein_seqs=encoded_seqs)
                 pyDSP.encode_seqs()
                 X_aai = pd.DataFrame(pyDSP.spectrum_encoding)
@@ -534,7 +530,7 @@ class Encoding(PySAR):
                 X_aai = pd.DataFrame(encoded_seqs)
 
             #iterate through all descriptors
-            for descr in tqdm(all_descriptors, leave=False,unit=" descriptor",desc="Descriptors"):
+            for descr in tqdm(all_descriptors, leave=False, unit=" descriptor", desc="Descriptors"):
 
                 #reset descriptor DF and list
                 desc_ = pd.DataFrame()
@@ -547,7 +543,7 @@ class Encoding(PySAR):
                         descriptor_list.append(getattr(desc, de)) #get descriptor attribute
                         # descriptor_list.append(desc.get_descriptor_encoding(de))
                         descriptor_group_.append(desc.descriptor_groups[de])
-                    desc_ = pd.concat(descriptor_list,axis=1) # check if this is needed here
+                    desc_ = pd.concat(descriptor_list, axis=1) # check if this is needed here
 
                 #if only using 1 descriptor
                 else:
@@ -555,7 +551,8 @@ class Encoding(PySAR):
                     # desc_ = desc.get_descriptor_encoding(descr)
                     descriptor_group_.append(desc.descriptor_groups[descr])
 
-                X = desc_   #set training data to descriptor dataframe
+                # X = desc_   #set training data to descriptor dataframe
+                X = pd.concat([desc_, X_aai], axis=1)
 
                 #get protein activity values
                 Y  = self.get_activity()
@@ -626,7 +623,7 @@ class Encoding(PySAR):
             sort_by = 'R2'
 
         #sort results by R2
-        aai_desc_metrics_df_ = aai_desc_metrics_df_.sort_values(by=[sort_by],ascending=False)
+        aai_desc_metrics_df_ = aai_desc_metrics_df_.sort_values(by=[sort_by], ascending=False)
 
         #set save path according to the descriptor combinations type
         if desc_combo == 2:
@@ -644,11 +641,9 @@ class Encoding(PySAR):
 ################################################################################
 
     def __str__(self):
-        return "Instance of Encoding Class with attribute values: \
-                \nDataset: {}\n, Activity: {}\n,Algorithm: {}\n Parameters: {}\nTest Split: {}\n".format(
-                    self.dataset, self.activity,self.algorithm,\
-                    self.parameters, self.test_split
+        return "Instance of Encoding Class with attribute values: Dataset: {}, Activity: {}, Algorithm: {}, Parameters: {}, Test Split: {}.".format(
+                    self.dataset, self.activity,self.algorithm, self.parameters, self.test_split
         )
 
     def __repr__(self):
-        return "<Encoding: {} >".format(self)
+        return "<{} >".format(self)
